import fs_implementation from 'node:fs';
import process from 'node:process';
import path from 'node:path';

function sync_impl$g(fs, path, mode) {
	fs.chmodSync(path, mode);
}

async function async_impl$g(fs, path, mode) {
	await fs.promises.chmod(path, mode);
}

var chmod = {sync_impl: sync_impl$g, async_impl: async_impl$g};

function sync_impl$f(fs, path, uid, gid) {
	fs.chownSync(path, uid, gid);
}

async function async_impl$f(fs, path, uid, gid) {
	await fs.promises.chown(path, uid, gid);
}

var chown = {sync_impl: sync_impl$f, async_impl: async_impl$f};

function sync_impl$e(fs, src, dest) {
	fs.copyFileSync(src, dest);
}

async function async_impl$e(fs, src, dest) {
	await fs.promises.copyFile(src, dest);
}

var copyFile = {sync_impl: sync_impl$e, async_impl: async_impl$e};

function sync_impl$d(fs, path) {
	return fs.lstatSync(path)
}

async function async_impl$d(fs, path) {
	return await fs.promises.lstat(path)
}

var lstat = {sync_impl: sync_impl$d, async_impl: async_impl$d};

function sync_impl$c(fs, path, options = null) {
	if (options === null) {
		fs.mkdirSync(path);
	} else {
		fs.mkdirSync(path, options);
	}
}

async function async_impl$c(fs, path, options = null) {
	if (options === null) {
		await fs.promises.mkdir(path);
	} else {
		await fs.promises.mkdir(path, options);
	}
}

var mkdir = {sync_impl: sync_impl$c, async_impl: async_impl$c};

function sync_impl$b(fs, path) {
	const handle = fs.opendirSync(path);

	return {
		read() {
			return handle.readSync()
		},
		close() {
			handle.closeSync();
		}
	}
}

async function async_impl$b(fs, path) {
	const handle = await fs.promises.opendir(path);

	return {
		async read() {
			return await handle.read()
		},
		async close() {
			await handle.close();
		}
	}
}

var openDirectory = {sync_impl: sync_impl$b, async_impl: async_impl$b};

function sync_impl$a(fs, path, flags = "r", mode = 0o666) {
	const fd = fs.openSync(path, flags, mode);

	return {
		read(buffer) {
			return fs.readSync(fd, buffer)
		},
		write(buffer) {
			return fs.writeSync(fd, buffer)
		},
		close() {
			fs.closeSync(fd);
		}
	}
}

async function async_impl$a(fs, path, flags = "r", mode = 0o666) {
	const handle = await fs.promises.open(path, flags, mode);

	return {
		async read(buffer) {
			const {bytesRead} = await handle.read(buffer, 0, buffer.byteLength, null);

			return bytesRead
		},
		async write(buffer) {
			const {bytesWritten} = await handle.write(buffer);

			return bytesWritten
		},
		async close() {
			await handle.close();
		}
	}
}

var openFile = {sync_impl: sync_impl$a, async_impl: async_impl$a};

function sync_impl$9(fs, path) {
	return fs.readFileSync(path)
}

async function async_impl$9(fs, path) {
	return await fs.promises.readFile(path)
}

var readFile = {sync_impl: sync_impl$9, async_impl: async_impl$9};

function sync_impl$8(fs, path) {
	return fs.readdirSync(path)
}

async function async_impl$8(fs, path) {
	return await fs.promises.readdir(path)
}

var readdir = {sync_impl: sync_impl$8, async_impl: async_impl$8};

function sync_impl$7(fs, path) {
	return fs.readlinkSync(path)
}

async function async_impl$7(fs, path) {
	return await fs.promises.readlink(path)
}

var readlink = {sync_impl: sync_impl$7, async_impl: async_impl$7};

function sync_impl$6(fs, path) {
	return fs.realpathSync(path)
}

async function async_impl$6(fs, path) {
	return await fs.promises.realpath(path)
}

var realpath = {sync_impl: sync_impl$6, async_impl: async_impl$6};

function sync_impl$5(fs, src, dest) {
	fs.renameSync(src, dest);
}

async function async_impl$5(fs, src, dest) {
	await fs.promises.rename(src, dest);
}

var rename = {sync_impl: sync_impl$5, async_impl: async_impl$5};

function sync_impl$4(fs, path) {
	fs.rmdirSync(path);
}

async function async_impl$4(fs, path) {
	await fs.promises.rmdir(path);
}

var rmdir = {sync_impl: sync_impl$4, async_impl: async_impl$4};

function sync_impl$3(fs, path) {
	return fs.statSync(path)
}

async function async_impl$3(fs, path) {
	return await fs.promises.stat(path)
}

var stat = {sync_impl: sync_impl$3, async_impl: async_impl$3};

function sync_impl$2(fs, target, path) {
	fs.symlinkSync(target, path);
}

async function async_impl$2(fs, target, path) {
	await fs.promises.symlink(target, path);
}

var symlink = {sync_impl: sync_impl$2, async_impl: async_impl$2};

function sync_impl$1(fs, path) {
	fs.unlinkSync(path);
}

async function async_impl$1(fs, path) {
	await fs.promises.unlink(path);
}

var unlink = {sync_impl: sync_impl$1, async_impl: async_impl$1};

function sync_impl$h(fs, path, data) {
	return fs.writeFileSync(path, data)
}

async function async_impl$h(fs, path, data) {
	return await fs.promises.writeFile(path, data)
}

var writeFile = {sync_impl: sync_impl$h, async_impl: async_impl$h};

/* warning: this file was automatically generated */

var methods = {
    chmod,
    chown,
    copyFile,
    lstat,
    mkdir,
    openDirectory,
    openFile,
    readFile,
    readdir,
    readlink,
    realpath,
    rename,
    rmdir,
    stat,
    symlink,
    unlink,
    writeFile,
};

function createFSObject({
	sync = false
} = {}) {
	let fs_object = {};

	for (const method_name in methods) {
		const method = methods[method_name];
		const method_implementation = sync ? method.sync_impl : method.async_impl;

		fs_object[method_name] = (...args) => {

			if (process.env) {
				if ("ANIO_NODE_FS_API_DEBUG" in process.env) {
					console.error(`anio-node-fs-api ${method_name}${sync ? "Sync" : "Async"} with args`, args);
				}
			}

			return method_implementation(fs_implementation, ...args)
		};
	}

	return fs_object
}

function scandir$1(fs_object, root_dir, relative_entry_dir, options) {
	const entries = fs_object.readdir(
		path.join(root_dir, relative_entry_dir)
	);

	for (const entry of entries) {
		const absolute_path = path.join(root_dir, relative_entry_dir, entry);
		const relative_path = path.join(relative_entry_dir, entry);
		const stats = fs_object.lstat(absolute_path);

		let type = "file";

		if (stats.isSymbolicLink()) {
			type = "link";
		} else if (stats.isDirectory()) {
			type = "dir";
		}

		const handle_current_entry = () => {
			const data = {type, relative_path, absolute_path};

			if (typeof options.filter === "function") {
				const keep = options.filter(data);

				if (keep !== true) return
			}

			if (typeof options.callback === "function") {
				options.callback(data);

				return
			}

			options.entries.push(data);
		};

		const recurse = () => {
			if (type !== "dir") return

			scandir$1(fs_object, root_dir, relative_path, options);
		};

		if (options.reverse === true) recurse();

		handle_current_entry();

		// written this way so "if statement" has same length as options.reverse === true
		if (options.reverse !== true) recurse();
	}
}

function sync_impl(fs_object, root_dir, {
	callback = null,
	reverse = false,
	filter = null
} = {}) {
	let entries = [];
	const options = {callback, reverse, filter, entries};
	const resolved_root_path = fs_object.realpath(root_dir);

	scandir$1(fs_object, resolved_root_path, ".", options);

	return typeof callback === "function" ? null : entries
}

async function scandir(fs_object, root_dir, relative_entry_dir, options) {
	const entries = await fs_object.readdir(
		path.join(root_dir, relative_entry_dir)
	);

	for (const entry of entries) {
		const absolute_path = path.join(root_dir, relative_entry_dir, entry);
		const relative_path = path.join(relative_entry_dir, entry);
		const stats = await fs_object.lstat(absolute_path);

		let type = "file";

		if (stats.isSymbolicLink()) {
			type = "link";
		} else if (stats.isDirectory()) {
			type = "dir";
		}

		const handle_current_entry = async () => {
			const data = {type, relative_path, absolute_path};

			if (typeof options.filter === "function") {
				const keep = await options.filter(data);

				if (keep !== true) return
			}

			if (typeof options.callback === "function") {
				await options.callback(data);

				return
			}

			options.entries.push(data);
		};

		const recurse = async () => {
			if (type !== "dir") return

			await scandir(fs_object, root_dir, relative_path, options);
		};

		if (options.reverse === true) await recurse();

		await handle_current_entry();

		// written this way so "if statement" has same length as options.reverse === true
		if (options.reverse !== true) await recurse();
	}
}

async function async_impl(fs_object, root_dir, {
	callback = null,
	reverse = false,
	filter = null
} = {}) {
	let entries = [];
	const options = {callback, reverse, filter, entries};
	const resolved_root_path = await fs_object.realpath(root_dir);

	await scandir(fs_object, resolved_root_path, ".", options);

	return typeof callback === "function" ? null : entries
}

const async_fs = createFSObject({sync: false});
const sync_fs = createFSObject({sync: true});

function nodeFsScandir(root_dir, options = {}) {
	return async_impl(async_fs, root_dir, options)
}

nodeFsScandir.sync = function(root_dir, options = {}) {
	return sync_impl(sync_fs, root_dir, options)
};

export { nodeFsScandir as default };
